/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Boiler class


*******************************************************************************
******************************************************************************/

#include "Boiler.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_BOILERS 16

//------------------------------------------------------------------------------
//	Boiler Enemy
//------------------------------------------------------------------------------
Boiler::Boiler()
{
	Frame = 0;
	Active = false;	
	Hp = 1; 
	DrawMode = EN_NORMAL;
	
}

Boiler::~Boiler()
{
}


void Boiler::Update()
{
	const s32 Delta = BRAD_PI/64;
	
	Counter++;
	Angle += Delta;
	
	
	if( (Angle & (BRAD_PI-1)) == 0 )
	{
		Radius = 20 + qran_range(0,60);
		
	}
	
	if(Counter > 60*3)
	{
		if( (Angle & (BRAD_PI-1)) == 0 )
		{
			if (qran_range(0,255) > 200) Dx = -Dx;
		}
	
		if( x > (256 << 12) ) Dx = -Dx;
	}

	x += Dx;
	Dy = -abs(sinLerp( Angle )) * Radius;
	y = Sy + Dy;
	
	
		
	if( x < MIN_X )
	{
		Active = false;
		x = 300 << 12;
		y = 96 << 12;
	}

	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) & 3;
	}
	
	if( Active ) Targets::Instance()->AddCoord( x, y );
	

}


	
//------------------------------------------------------------------------------
//	Boiler Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
BoilerContainer::BoilerContainer()
{
}
	
BoilerContainer::~BoilerContainer()
{
	Boilers.clear();				// clear the bullets
}


void BoilerContainer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Boilers.resize( MAX_BOILERS/2 );			// 1/2 of max (speed of iteration)
	Boilers.reserve( MAX_BOILERS );		// reserve max for speed.  No need to do this for enemies
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void BoilerContainer::UpdateEntities()
{
	
	std::vector<Boiler>::iterator iter;
	for( iter = Boilers.begin(); iter != Boilers.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void BoilerContainer::DrawEntities()
{
	
	glColor( RGB15(31,31,31) );
	numEntities = 0;
	std::vector<Boiler>::iterator iter;
	for( iter = Boilers.begin(); iter != Boilers.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
					      GL_FLIP_NONE, &Images[iter->Frame+52] );
				glColor( RGB15(31,31,31) );
			}
			else
			{
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
						  GL_FLIP_NONE, &Images[iter->Frame+52] );
			}
			
		}
	}
	
}

bool BoilerContainer::Collide( StarFighter &Star )
{
	std::vector<Boiler>::iterator iter;
	for( iter = Boilers.begin(); iter != Boilers.end(); ++iter )
	{
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 2 );
					Star.IncreaseGauge(15);
					iter->Hp--;
					if( (iter->Hp & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( iter->Hp < 0 )
					{
						iter->Active = false;
						Star.Score += 33;
						Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
						// Add some nice explosions
						Explosion::Instance()->Spawn( iter->x >> 12,
													  iter->y >> 12,
													   0,
													   0,
													   Explode::SMALL_2
													 );
							
						iter->x = 600 <<12;
						iter->y = 96 << 12;
						
					}
					if( iter->DrawMode == EN_NORMAL )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				return true;
			}
		}
		
		// missiles if after star enemy is still active
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			
			if( iter->x < ( 256 << 12 ) )
			{
			
				// Missiles
				for( int i = 0; i < MAX_MISSILES; i++)
				{
					HomingMissile Missile= Star.GetMissile(i);
					
					if( Missile.IsActive() )
					{
						int cx = (iter->x >> 12) - (Missile.GetX() >> 12);
						int cy = (iter->y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 4 );
							Star.DestroyMissile(i);
							iter->Hp--;
							if( iter->Hp < 0 )
							{
								iter->Active = false;
								Star.Score += 33;
								Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
								// Add some nice explosions
								Explosion::Instance()->Spawn( iter->x >> 12,
															  iter->y >> 12,
															   0,
															   0,
															   Explode::SMALL_4
															 );
									
								iter->x = 600 <<12;
								iter->y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   // iter x < 
	
		}   // iter active
		
	}
	return false;
}



void BoilerContainer::KillAllEntities()
{
	std::vector<Boiler>::iterator iter;
	for( iter = Boilers.begin(); iter != Boilers.end(); ++iter )
	{
		if( iter->Active )
		{
			iter->Active = false;
			iter->x = 600 << 12;
			iter->y = 96 << 12;
		}
	}
	
}

//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void BoilerContainer::Spawn( int _x, s32 _speed, int _radius )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Boiler>::iterator iter;
	for( iter = Boilers.begin(); iter != Boilers.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->x = _x << 12;
			iter->Sy = (191-8) << 12;
			iter->Dx = _speed;
			iter->Radius = _radius;
			iter->Angle = qran_range(0, 8) * 2048;
			
			iter->Active = true;
			iter->Frame = 0;
			iter->Counter = 0;
			iter->Hp = 1;
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Boilers.size() < MAX_BOILERS) )
	{
		Boiler popcorn;
		popcorn.x = _x << 12;
		popcorn.Sy = (191-8) << 12;
		popcorn.Dx = _speed;
		popcorn.Radius = _radius;
		popcorn.Angle = qran_range(0, 8) * 2048;
			
		popcorn.Active = true;
		popcorn.Frame = 0;
		popcorn.Counter = 0;
		popcorn.Hp = 1;
		Boilers.push_back(popcorn);
	}
	
}




